def pytest_configure():
    from django.conf import settings

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3',
                               'NAME': ':memory:'}},
        SITE_ID=1,
        SECRET_KEY='not very secret in tests',
        USE_I18N=True,
        USE_L10N=True,
        STATIC_URL='/static/',
        ROOT_URLCONF='tests.urls',
        TEMPLATE_LOADERS=(
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
        ),
        MIDDLEWARE_CLASSES=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
        ),
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.messages',
            'django.contrib.staticfiles',

            'rest_framework',
            'rest_framework.authtoken',
            'tests',
        ),
        PASSWORD_HASHERS=(
            'django.contrib.auth.hashers.SHA1PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
            'django.contrib.auth.hashers.BCryptPasswordHasher',
            'django.contrib.auth.hashers.MD5PasswordHasher',
            'django.contrib.auth.hashers.CryptPasswordHasher',
        ),
    )

    try:
        import oauth_provider  # NOQA
        import oauth2  # NOQA
    except ImportError:
        pass
    else:
        settings.INSTALLED_APPS += (
            'oauth_provider',
        )

    try:
        import provider  # NOQA
    except ImportError:
        pass
    else:
        settings.INSTALLED_APPS += (
            'provider',
            'provider.oauth2',
        )

    # guardian is optional
    try:
        import guardian  # NOQA
    except ImportError:
        pass
    else:
        settings.ANONYMOUS_USER_ID = -1
        settings.AUTHENTICATION_BACKENDS = (
            'django.contrib.auth.backends.ModelBackend',
            'guardian.backends.ObjectPermissionBackend',
        )
        settings.INSTALLED_APPS += (
            'guardian',
        )

    try:
        import django
        django.setup()
    except AttributeError:
        pass



"""
Force import of all modules in this package in order to get the standard test
runner to pick up the tests.  Yowzers.
"""
from __future__ import unicode_literals
import os
import django

modules = [filename.rsplit('.', 1)[0]
           for filename in os.listdir(os.path.dirname(__file__))
           if filename.endswith('.py') and not filename.startswith('_')]
__test__ = dict()

if django.VERSION < (1, 6):
    for module in modules:
        exec("from drf_routers.tests.%s import *" % module)



"""
Routers for nested resources.

Example:

    # urls.py

    from rest_framework_nested import routers

    router = routers.SimpleRouter()
    router.register(r'domains', DomainViewSet)

    domains_router = routers.NestedSimpleRouter(router, r'domains', lookup='domain')
    domains_router.register(r'nameservers', NameserverViewSet)

    url_patterns = patterns('',
        url(r'^', include(router.urls)),
            url(r'^', include(domains_router.urls)),
            )

        router = routers.DefaultRouter()
        router.register('users', UserViewSet, 'user')
        router.register('accounts', AccountViewSet, 'account')

        urlpatterns = router.urls
"""

from __future__ import unicode_literals

from rest_framework.routers import SimpleRouter, DefaultRouter
from rest_framework.views import APIView
from rest_framework.response import Response
from django.conf.urls import patterns, include, url
from django.core.urlresolvers import NoReverseMatch
from django.core.urlresolvers import reverse


class LookupMixin(object):
	"""
	Deprecated.

	No method override is needed since Django Rest Framework 2.4.
	"""


def reverseBase(request, view_name, absolute=False, urlconf=None, args=None, kwargs=None, prefix=None,
				current_app=None):
	if not request.resolver_match.namespace:
		namespace = ''
	else:
		namespace = request.resolver_match.namespace + ':'
	url = reverse(namespace + view_name, urlconf, args, kwargs, prefix, current_app);
	if absolute:
		url = request.build_absolute_uri(url)
	return url;


class GroupedRouter(DefaultRouter):
	"""
	The GroupedRouter router extends the DefaultRouter, add feature to include another
	GroupedRouter, NestedRouter or DefaultRounter to show in api root view
	"""
	include_root_view = True
	include_format_suffixes = True
	root_view_name = 'api-root'

	def __init__(self, root_view_name='api-root'):
		super(self.__class__, self).__init__();
		self.includeRouterList = []
		self.root_view_name = root_view_name

	def get_api_root_view(self):
		"""
		Return a view to use as the API root.
		"""
		api_root_dict = {}
		list_name = self.routes[0].name
		for prefix, viewset, basename in self.registry:
			api_root_dict[prefix] = basename, viewset  # list_name.format(basename=basename)
		routerSelf = self;
		class_name = self.root_view_name.replace('-', '_').capitalize()

		class APIRoot(APIView):
			_ignore_model_permissions = True

			def get(self, request, *args, **kwargs):
				ret = []
				for r in routerSelf.includeRouterList:
					try:
						ret += [{
							'name': r.root_view_name,
							'url': reverseBase(request, r.root_view_name, absolute=True),
						}]
					except NoReverseMatch:
						continue;

				for prefix in api_root_dict:
					basename, viewset = api_root_dict[prefix]
					for url, mapping, name, initkwargs in routerSelf.get_routes(viewset):
						correctName = name.format(basename=basename);

						reverseKwagrs = {};

						# suffix = initkwargs.get('suffix');
						# if suffix==u'List':
						# 	pass;
						# elif suffix==u'Instance':
						# 	reverseKwagrs = {'pk':1}

						if 'lookup' in url:
							reverseKwagrs = {'pk': 1}

						try:
							ret += [{
								'name': correctName,
								'methods': mapping,
								'url': reverseBase(request, correctName, absolute=True, kwargs=reverseKwagrs),
								# 'url': request.build_absolute_uri(request.get_full_path()+url[1:-1].format(prefix=prefix,lookup='1',trailing_slash='/')),
								# 'initKwagrs': str(initkwargs),

							}];
						except NoReverseMatch:
							continue;

				return Response(ret)

		APIRoot.__name__ = self.root_view_name;
		return APIRoot.as_view()

	def get_urls(self):
		urlpatterns = super(self.__class__, self).get_urls();
		for r in self.includeRouterList:
			urlpatterns.append(url('^' + r.root_view_name + '/', include(r.urls)));

		return urlpatterns;

	def include(self, r):
		""" Include another router to this route

		:param r: an instance of DefaultRouter
		:return:
		"""
		self.includeRouterList.append(r);


class NestedRouter(DefaultRouter):
	def __init__(self, parent_router, parent_prefix, *args, **kwargs):
		""" Create a NestedRouter nested within `parent_router`
		Args:

		parent_router: Parent router. Mayb be a simple router or another nested
			router.

		parent_prefix: The url prefix within parent_router under which the
			routes from this router should be nested.

		lookup:
			The regex variable that matches an instance of the parent-resource
			will be called '<lookup>_<parent-viewset.lookup_field>'
			In the example above, lookup=domain and the parent viewset looks up
			on 'pk' so the parent lookup regex will be 'domain_pk'.
			Default: 'nested_<n>' where <n> is 1+parent_router.nest_count

		"""
		self.parent_router = parent_router
		self.parent_prefix = parent_prefix
		self.nest_count = getattr(parent_router, 'nest_count', 0) + 1
		self.nest_prefix = kwargs.pop('lookup', 'nested_%i' % self.nest_count) + '_'
		super(NestedRouter, self).__init__(*args, **kwargs)
		parent_registry = [registered for registered in self.parent_router.registry if
						   registered[0] == self.parent_prefix]
		try:
			parent_registry = parent_registry[0]
			parent_prefix, parent_viewset, parent_basename = parent_registry
		except:
			raise RuntimeError('parent registered resource not found')

		nested_routes = []
		parent_lookup_regex = parent_router.get_lookup_regex(parent_viewset, self.nest_prefix)

		self.parent_regex = '{parent_prefix}/{parent_lookup_regex}/'.format(
			parent_prefix=parent_prefix,
			parent_lookup_regex=parent_lookup_regex
		)
		if hasattr(parent_router, 'parent_regex'):
			self.parent_regex = parent_router.parent_regex + self.parent_regex

		for route in self.routes:
			route_contents = route._asdict()

			# This will get passed through .format in a little bit, so we need
			# to escape it
			escaped_parent_regex = self.parent_regex.replace('{', '{{').replace('}', '}}')

			route_contents['url'] = route.url.replace('^', '^' + escaped_parent_regex)
			nested_routes.append(type(route)(**route_contents))

		self.routes = nested_routes



# -- coding: utf-8 --

# Apparently there is a python 2.6 issue where docstrings of imported view classes
# do not retain their encoding information even if a module has a proper
# encoding declaration at the top of its source file. Therefore for tests
# to catch unicode related errors, a mock view has to be declared in a separate
# module.

from rest_framework.views import APIView


# test strings snatched from http://www.columbia.edu/~fdc/utf8/,
# http://winrus.com/utf8-jap.htm and memory
UTF8_TEST_DOCSTRING = (
    'zażółć gęślą jaźń'
    'Sîne klâwen durh die wolken sint geslagen'
    'Τη γλώσσα μου έδωσαν ελληνική'
    'யாமறிந்த மொழிகளிலே தமிழ்மொழி'
    'На берегу пустынных волн'
    'てすと'
    'ｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃ'
)


class ViewWithNonASCIICharactersInDocstring(APIView):
    __doc__ = UTF8_TEST_DOCSTRING



"""
based upon https://github.com/alanjds/drf-nested-routers/issues/15
"""
from collections import namedtuple

from django.test import TestCase
from django.db import models
from rest_framework.viewsets import ModelViewSet
from drf_routers.routers import SimpleRouter, NestedRouter
from rest_framework.response import Response


QS = namedtuple('Queryset', ['model'])

class BasicModel(models.Model):
    name=models.CharField(max_length=255)
    class Meta:
        app_label = 'testapp'


try:
    from rest_framework.decorators import detail_route, list_route
except ImportError:
    pass
else:
    def map_by_name(iterable):
        ret = {}
        for item in iterable:
            ret[item.name] = item
        return ret

    class DetailRouteViewSet(ModelViewSet):
        model = BasicModel
        queryset = QS(BasicModel)

        @detail_route(methods=["post"])
        def set_password(self, request, pk=None):
            return Response({'hello': 'ok'})

    class ListRouteViewSet(ModelViewSet):
        model = BasicModel
        queryset = QS(BasicModel)

        @list_route()
        def recent_users(self, request, pk=None):
            return Response([{'hello': 'ok'}])

    class TestNestedSimpleRouter(TestCase):
        def setUp(self):
            self.router = SimpleRouter()
            self.router.register(r'detail', DetailRouteViewSet)
            self.detail_router = NestedRouter(
                self.router,
                r'detail',
                lookup='detail'
            )
            self.detail_router.register(r'list', ListRouteViewSet)
            self.list_router = NestedRouter(
                self.detail_router,
                r'list',
                lookup='list'
            )

        def test_dynamic_routes(self):
            self.assertFalse(hasattr(self.router, 'parent_regex'))
            urls = map_by_name(self.router.urls)
            self.assertEquals(
                urls['basicmodel-list'].regex.pattern, u'^detail/$'
            )
            self.assertEquals(
                urls['basicmodel-detail'].regex.pattern,
                u'^detail/(?P<pk>[^/.]+)/$'
            )
            self.assertEquals(
                urls['basicmodel-set-password'].regex.pattern,
                u'^detail/(?P<pk>[^/.]+)/set_password/$'
            )

        def test_nested_parent(self):
            self.assertEqual(
                self.detail_router.parent_regex,
                u'detail/(?P<detail_pk>[^/.]+)/'
            )
            urls = map_by_name(self.detail_router.urls)

            self.assertEquals(
                urls['basicmodel-list'].regex.pattern,
                u'^detail/(?P<detail_pk>[^/.]+)/list/$'
            )

            self.assertEquals(
                urls['basicmodel-recent-users'].regex.pattern,
                u'^detail/(?P<detail_pk>[^/.]+)/list/recent_users/$'
            )

            self.assertEquals(
                urls['basicmodel-detail'].regex.pattern,
                u'^detail/(?P<detail_pk>[^/.]+)/list/(?P<pk>[^/.]+)/$'
            )

        def test_nested_child(self):
            self.assertEqual(
                self.list_router.parent_regex,
                u'detail/(?P<detail_pk>[^/.]+)/list/(?P<list_pk>[^/.]+)/'
            )



"""
Serializer fields that deal with relationships with nested resources.

These fields allow you to specify the style that should be used to represent
model relationships with hyperlinks.
"""
from __future__ import unicode_literals

import rest_framework.relations


class NestedHyperlinkedRelatedField(rest_framework.relations.HyperlinkedRelatedField):
    lookup_field = 'pk'
    parent_lookup_field = 'parent'
    parent_lookup_related_field = 'pk'

    def __init__(self, *args, **kwargs):
        self.parent_lookup_field = kwargs.pop('parent_lookup_field', self.parent_lookup_field)
        self.parent_lookup_url_kwarg = kwargs.pop('parent_lookup_url_kwarg', self.parent_lookup_field)
        self.parent_lookup_related_field = kwargs.pop('parent_lookup_related_field', self.parent_lookup_related_field)
        super(NestedHyperlinkedRelatedField, self).__init__(*args, **kwargs)

    def get_url(self, obj, view_name, request, format):
        """
        Given an object, return the URL that hyperlinks to the object.

        May raise a `NoReverseMatch` if the `view_name` and `lookup_field`
        attributes are not configured to correctly match the URL conf.
        """
        # Unsaved objects will not yet have a valid URL.
        if hasattr(obj, 'pk') and obj.pk is None:
            return None
        pk_url_kwarg = 'pk'

        lookup_value = getattr(obj, self.lookup_field)
        parent_lookup_object = getattr(obj, self.parent_lookup_field)
        parent_lookup_value = getattr(
            parent_lookup_object,
            self.parent_lookup_related_field
        ) if parent_lookup_object else None

        kwargs = {
            self.lookup_url_kwarg: lookup_value,
            self.parent_lookup_url_kwarg: parent_lookup_value,
        }
        return self.reverse(view_name, kwargs=kwargs, request=request, format=format)

    def get_object(self, view_name, view_args, view_kwargs):
        """
        Return the object corresponding to a matched URL.

        Takes the matched URL conf arguments, and should return an
        object instance, or raise an `ObjectDoesNotExist` exception.
        """
        lookup_value = view_kwargs[self.lookup_url_kwarg]
        parent_lookup_value = view_kwargs[self.parent_lookup_url_kwarg]
        lookup_kwargs = {
            self.lookup_field: lookup_value,
            self.parent_lookup_field: parent_lookup_value,
        }
        return self.get_queryset().get(**lookup_kwargs)



"""
based upon https://github.com/alanjds/drf-nested-routers/issues/15
"""
from collections import namedtuple

from django.db import models
from django.test import TestCase
from rest_framework.viewsets import ModelViewSet
from drf_routers.routers import SimpleRouter, NestedRouter


QS = namedtuple('Queryset', ['model'])


class A(models.Model):
    name=models.CharField(max_length=255)
class B(models.Model):
    name=models.CharField(max_length=255)
    parent=models.ForeignKey(A)
class C(models.Model):
    name=models.CharField(max_length=255)
    parent=models.ForeignKey(B)

class AViewSet(ModelViewSet):
    lookup_value_regex = '[0-9a-f]{32}'
    model = A
    queryset = QS(A)

class BViewSet(ModelViewSet):
    model = B
    queryset = QS(B)

class CViewSet(ModelViewSet):
    model = C
    queryset = QS(C)

class TestNestedSimpleRouter(TestCase):
    def setUp(self):
        self.router = SimpleRouter()
        self.router.register(r'a', AViewSet)
        self.a_router = NestedRouter(self.router, r'a', lookup='a')
        self.a_router.register(r'b', BViewSet)
        self.b_router = NestedRouter(self.a_router, r'b', lookup='b')
        self.b_router.register(r'c', CViewSet)

    def test_recursive_nested_simple_routers(self):
        self.assertFalse(hasattr(self.router, 'parent_regex'))
        urls = self.router.urls
        self.assertEquals(len(urls), 2)
        self.assertEquals(urls[0].regex.pattern, u'^a/$')
        self.assertEquals(urls[1].regex.pattern, u'^a/(?P<pk>[0-9a-f]{32})/$')

        self.assertEqual(self.a_router.parent_regex, u'a/(?P<a_pk>[0-9a-f]{32})/')
        urls = self.a_router.urls
        self.assertEquals(len(urls), 2)
        self.assertEquals(urls[0].regex.pattern, u'^a/(?P<a_pk>[0-9a-f]{32})/b/$')
        self.assertEquals(urls[1].regex.pattern, u'^a/(?P<a_pk>[0-9a-f]{32})/b/(?P<pk>[^/.]+)/$')

        self.assertEqual(self.b_router.parent_regex, u'a/(?P<a_pk>[0-9a-f]{32})/b/(?P<b_pk>[^/.]+)/')
        urls = self.b_router.urls
        self.assertEquals(len(urls), 2)
        self.assertEquals(urls[0].regex.pattern, u'^a/(?P<a_pk>[0-9a-f]{32})/b/(?P<b_pk>[^/.]+)/c/$')
        self.assertEquals(urls[1].regex.pattern, u'^a/(?P<a_pk>[0-9a-f]{32})/b/(?P<b_pk>[^/.]+)/c/(?P<pk>[^/.]+)/$')



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#  setup.py
#
#
#  Created by TVA on 4/19/16.
#  Copyright (c) 2016 django-query-execfile. All rights reserved.
#

from setuptools import setup, find_packages

__version__ = '0.1'

setup(
	name='drf-routers',
	version=__version__,
	description='Grouped/Nested resources for the Django Rest Framework',
	author='V.Anh Tran',
	author_email='tranvietanh1991@gmail.com',
	license="MIT",
	url='https://github.com/tranvietanh1991/drf-routers',  # use the URL to the github repo
	download_url='https://github.com/tranvietanh1991/drf-routers/archive/master.zip',  # source code download
	packages=find_packages(exclude=['*.tests', '*.tests.*']),
	include_package_data=True,
	install_requires=[
		"Django>=1.7",
	],
	keywords=['django', 'rest', 'drf', 'django-rest-framwork', 'router', 'nested', 'grouped'],  # arbitrary keywords
	classifiers=[
		"Development Status :: 4 - Beta",
		"Environment :: Web Environment",
		"Framework :: Django",
		"Intended Audience :: Developers",
		"Intended Audience :: System Administrators",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
		"Programming Language :: Python :: 2",
		# "Programming Language :: Python :: 3",
		"Topic :: Software Development :: Libraries :: Python Modules",
	],
)


from __future__ import unicode_literals
from django.db import models
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers


def foobar():
    return 'foobar'


class CustomField(models.CharField):

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 12
        super(CustomField, self).__init__(*args, **kwargs)


class RESTFrameworkModel(models.Model):
    """
    Base for test models that sets app_label, so they play nicely.
    """
    class Meta:
        app_label = 'tests'
        abstract = True


class HasPositiveIntegerAsChoice(RESTFrameworkModel):
    some_choices = ((1, 'A'), (2, 'B'), (3, 'C'))
    some_integer = models.PositiveIntegerField(choices=some_choices)


class Anchor(RESTFrameworkModel):
    text = models.CharField(max_length=100, default='anchor')


class BasicModel(RESTFrameworkModel):
    text = models.CharField(max_length=100, verbose_name=_("Text comes here"), help_text=_("Text description."))


class SlugBasedModel(RESTFrameworkModel):
    text = models.CharField(max_length=100)
    slug = models.SlugField(max_length=32)


class DefaultValueModel(RESTFrameworkModel):
    text = models.CharField(default='foobar', max_length=100)
    extra = models.CharField(blank=True, null=True, max_length=100)


class CallableDefaultValueModel(RESTFrameworkModel):
    text = models.CharField(default=foobar, max_length=100)


class ManyToManyModel(RESTFrameworkModel):
    rel = models.ManyToManyField(Anchor, help_text='Some help text.')


class ReadOnlyManyToManyModel(RESTFrameworkModel):
    text = models.CharField(max_length=100, default='anchor')
    rel = models.ManyToManyField(Anchor)


# Model for regression test for #285

class Comment(RESTFrameworkModel):
    email = models.EmailField()
    content = models.CharField(max_length=200)
    created = models.DateTimeField(auto_now_add=True)


class ActionItem(RESTFrameworkModel):
    title = models.CharField(max_length=200)
    done = models.BooleanField(default=False)
    info = CustomField(default='---', max_length=12)


# Models for reverse relations
class Person(RESTFrameworkModel):
    name = models.CharField(max_length=10)
    age = models.IntegerField(null=True, blank=True)

    @property
    def info(self):
        return {
            'name': self.name,
            'age': self.age,
        }


class BlogPost(RESTFrameworkModel):
    title = models.CharField(max_length=100)
    writer = models.ForeignKey(Person, null=True, blank=True)

    def get_first_comment(self):
        return self.blogpostcomment_set.all()[0]


class BlogPostComment(RESTFrameworkModel):
    text = models.TextField()
    blog_post = models.ForeignKey(BlogPost)


class Album(RESTFrameworkModel):
    title = models.CharField(max_length=100, unique=True)


class Photo(RESTFrameworkModel):
    description = models.TextField()
    album = models.ForeignKey(Album)


# Model for issue #324
class BlankFieldModel(RESTFrameworkModel):
    title = models.CharField(max_length=100, blank=True, null=False)


# Model for issue #380
class OptionalRelationModel(RESTFrameworkModel):
    other = models.ForeignKey('OptionalRelationModel', blank=True, null=True)


# Model for RegexField
class Book(RESTFrameworkModel):
    isbn = models.CharField(max_length=13)


# Models for relations tests
# ManyToMany
class ManyToManyTarget(RESTFrameworkModel):
    name = models.CharField(max_length=100)


class ManyToManySource(RESTFrameworkModel):
    name = models.CharField(max_length=100)
    targets = models.ManyToManyField(ManyToManyTarget, related_name='sources')


# ForeignKey
class ForeignKeyTarget(RESTFrameworkModel):
    name = models.CharField(max_length=100)


class ForeignKeySource(RESTFrameworkModel):
    name = models.CharField(max_length=100)
    target = models.ForeignKey(ForeignKeyTarget, related_name='sources')


# Nullable ForeignKey
class NullableForeignKeySource(RESTFrameworkModel):
    name = models.CharField(max_length=100)
    target = models.ForeignKey(ForeignKeyTarget, null=True, blank=True,
                               related_name='nullable_sources')


# OneToOne
class OneToOneTarget(RESTFrameworkModel):
    name = models.CharField(max_length=100)


class NullableOneToOneSource(RESTFrameworkModel):
    name = models.CharField(max_length=100)
    target = models.OneToOneField(OneToOneTarget, null=True, blank=True,
                                  related_name='nullable_source')


# Serializer used to test BasicModel
class BasicModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = BasicModel
